@@ -13,13 +13,17 @@
 #include <termios.h>
 #include <time.h>
 #include <math.h>
+#include <fcntl.h>
 #ifndef S_SPLINT_S
 #include <unistd.h>
 #include <sys/socket.h>
+#include <net/if.h>
+#include <sys/ioctl.h>
 #endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 #if defined(NMEA2000_ENABLE)
+#include "driver_nmea2000.h"
 #include "bits.h"
 
 #ifndef S_SPLINT_S
@@ -44,8 +48,9 @@ typedef struct PGN
 FILE *logFile = NULL;
 #endif /* of if LOG_FILE */
 
-static int print_data(unsigned char *buffer, int len, PGN *pgn)
+static void print_data(unsigned char *buffer, int len, PGN *pgn)
 {
+#ifdef LIBGPS_DEBUG
     /*@-bufferoverflowhigh@*/
     if ((libgps_debuglevel >= LOG_IO) != 0) {
 	int   l1, l2, ptr;
@@ -67,7 +72,7 @@ static int print_data(unsigned char *buffer, int len, PGN *pgn)
         gpsd_report(LOG_IO,"%s\n", bu);
     }
     /*@+bufferoverflowhigh@*/
-    return(0);
+#endif
 }
 
 static gps_mask_t get_mode(struct gps_device_t *session)
@@ -84,7 +89,7 @@ static gps_mask_t get_mode(struct gps_device_t *session)
 
 static gps_mask_t hnd_059392(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -92,7 +97,7 @@ static gps_mask_t hnd_059392(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_060928(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -100,7 +105,7 @@ static gps_mask_t hnd_060928(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_126208(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -108,7 +113,7 @@ static gps_mask_t hnd_126208(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_126464(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -116,7 +121,7 @@ static gps_mask_t hnd_126464(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_126996(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -124,7 +129,7 @@ static gps_mask_t hnd_126996(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129025(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
     /*@-type@*//* splint has a bug here */
@@ -140,7 +145,7 @@ static gps_mask_t hnd_129025(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129026(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
     session->driver.nmea2000.sid[0]  =  bu[0];
@@ -161,10 +166,10 @@ static gps_mask_t hnd_126992(unsigned char *bu, int len, PGN *pgn, struct gps_de
     //uint8_t        sid;
     //uint8_t        source;
 
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
     //sid        = bu[0];
@@ -187,7 +192,7 @@ static gps_mask_t hnd_129539(unsigned char *bu, int len, PGN *pgn, struct gps_de
 {
     gps_mask_t mask;
 
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
     mask                             = 0;
@@ -222,7 +227,7 @@ static gps_mask_t hnd_129540(unsigned char *bu, int len, PGN *pgn, struct gps_de
 {
     int         l1, l2;
 
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
     session->driver.nmea2000.sid[2]           = bu[0];
@@ -261,7 +266,7 @@ static gps_mask_t hnd_129029(unsigned char *bu, int len, PGN *pgn, struct gps_de
 {
     gps_mask_t mask;
 
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
 
     mask                             = 0;
@@ -326,7 +331,7 @@ static gps_mask_t hnd_129029(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129038(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -334,7 +339,7 @@ static gps_mask_t hnd_129038(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129039(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -342,7 +347,7 @@ static gps_mask_t hnd_129039(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129040(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -350,7 +355,7 @@ static gps_mask_t hnd_129040(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129794(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -358,7 +363,7 @@ static gps_mask_t hnd_129794(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129798(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -366,7 +371,7 @@ static gps_mask_t hnd_129798(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129802(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -374,7 +379,7 @@ static gps_mask_t hnd_129802(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129809(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -382,7 +387,7 @@ static gps_mask_t hnd_129809(unsigned char *bu, int len, PGN *pgn, struct gps_de
 
 static gps_mask_t hnd_129810(unsigned char *bu, int len, PGN *pgn, struct gps_device_t *session)
 {
-    (void)print_data(bu, len, pgn);
+    print_data(bu, len, pgn);
     gpsd_report(LOG_DATA, "pgn %6d(%3d):\n", pgn->pgn, session->driver.nmea2000.unit);
     return(0);
 }
@@ -629,6 +634,63 @@ static gps_mask_t nmea2000_parse_input(struct gps_device_t *session)
 
 /*@+nullassign@*/
 
+#ifndef S_SPLINT_S
+
+int nmea2000_open(struct gps_device_t *session)
+{
+    char interface_name[strlen(session->gpsdata.dev.path)];
+    socket_t sock;
+    int status;
+    struct ifreq ifr;
+    struct sockaddr_can addr;
+
+    session->gpsdata.gps_fd = -1;
+    /* Create the socket */
+    sock = socket(PF_CAN, SOCK_RAW, CAN_RAW);
+ 
+    if (sock == -1) {
+        gpsd_report(LOG_ERROR, "NMEA2000 open: can not get socket.\n");
+	return -1;
+    }
+
+    status = fcntl(sock, F_SETFL, O_NONBLOCK);
+    if (status != 0) {
+        gpsd_report(LOG_ERROR, "NMEA2000 open: can not set socket to O_NONBLOCK.\n");
+	close(sock);
+	return -1;
+    }
+
+    (void)strlcpy(interface_name, session->gpsdata.dev.path + 11, sizeof(interface_name));
+    /* Locate the interface you wish to use */
+    strlcpy(ifr.ifr_name, interface_name, sizeof(ifr.ifr_name));
+    status = ioctl(sock, SIOCGIFINDEX, &ifr); /* ifr.ifr_ifindex gets filled 
+					       * with that device's index */
+
+    if (status != 0) {
+        gpsd_report(LOG_ERROR, "NMEA2000 open: can not find CAN device.\n");
+	close(sock);
+	return -1;
+    }
+
+    /* Select that CAN interface, and bind the socket to it. */
+    addr.can_family = AF_CAN;
+    addr.can_ifindex = ifr.ifr_ifindex;
+    status = bind(sock, (struct sockaddr*)&addr, sizeof(addr) );
+    if (status != 0) {
+        gpsd_report(LOG_ERROR, "NMEA2000 open: bind failed.\n");
+	close(sock);
+	return -1;
+    }
+
+    gpsd_switch_driver(session, "NMEA2000");
+    session->gpsdata.gps_fd = sock;
+    session->sourcetype = source_can;
+    session->servicetype = service_sensor;
+    return session->gpsdata.gps_fd;
+}
+
+#endif /* of ifndef S_SPLINT_S */
+
 /* *INDENT-OFF* */
 const struct gps_type_t nmea2000 = {
     .type_name      = "NMEA2000",       /* full name of type */
