@@ -43,7 +43,7 @@ extern "C" {
 #define MAXTAGLEN	8	/* maximum length of sentence tag name */
 #define MAXCHANNELS	72	/* must be > 12 GPS + 12 GLONASS + 2 WAAS */
 #define GPS_PRNMAX	32	/* above this number are SBAS satellites */
-#define GPS_PATH_MAX	64	/* dev files usually have short names */
+#define GPS_PATH_MAX	128	/* dev files usually have short names */
 #define MAXUSERDEVS	4	/* max devices per user */
 
 /* 
@@ -850,12 +850,12 @@ typedef /*@unsignedintegraltype@*/ unsigned long long gps_mask_t;
 /* N/A values and scaling constant for 25/24 bit lon/lat pairs */
 #define AIS_LON3_NOT_AVAILABLE	181000
 #define AIS_LAT3_NOT_AVAILABLE	91000
-#define AIS_LATLON3_SCALE	60000.0
+#define AIS_LATLON3_DIV	60000.0
 
 /* N/A values and scaling constant for 28/27 bit lon/lat pairs */
 #define AIS_LON4_NOT_AVAILABLE	1810000
 #define AIS_LAT4_NOT_AVAILABLE	910000
-#define AIS_LATLON4_SCALE	600000.0
+#define AIS_LATLON4_DIV	600000.0
 
 struct route_info {
     unsigned int linkage;	/* Message Linkage ID */
@@ -890,7 +890,7 @@ struct ais_t
 #define AIS_SPEED_NOT_AVAILABLE	1023
 #define AIS_SPEED_FAST_MOVER	1022		/* >= 102.2 knots */
 	    bool accuracy;			/* position accuracy */
-#define AIS_LATLON_SCALE	600000.0
+#define AIS_LATLON_DIV	600000.0
 	    int lon;				/* longitude */
 #define AIS_LON_NOT_AVAILABLE	0x6791AC0
 	    int lat;				/* latitude */
@@ -965,6 +965,17 @@ struct ais_t
 	    size_t bitcount;		/* bit count of the data */
 	    union {
 		char bitdata[(AIS_TYPE6_BINARY_MAX + 7) / 8];
+		/* GLA - AtoN monitoring data (UK/ROI) */
+		struct {
+		    unsigned int ana_int;       /* Analogue (internal) */
+		    unsigned int ana_ext1;      /* Analogue (external #1) */
+		    unsigned int ana_ext2;      /* Analogue (external #2) */
+		    unsigned int racon; /* RACON status */
+		    unsigned int light; /* Light status */
+		    bool alarm; /* Health alarm*/
+		    unsigned int stat_ext;      /* Status bits (external) */
+		    bool off_pos;    /* Off position status */
+		} dac235fid10;
 		/* IMO236 - Dangerous Cargo Indication */
 		struct {
 		    char lastport[5+1];		/* Last Port Of Call */
@@ -1044,6 +1055,93 @@ struct ais_t
 		    signed int berth_lon;	/* Longitude */
 		    signed int berth_lat;	/* Latitude */
 		} dac1fid20;
+		/* IMO289 - Weather observation report from ship */
+		/*** WORK IN PROGRESS - NOT YET DECODED ***/
+		struct {
+		    bool wmo;			/* true if WMO variant */
+		    union {
+			struct {
+			    char location[20+1];	/* Location */
+			    signed int lon;		/* Longitude */
+			    signed int lat;		/* Latitude */
+			    unsigned int day;		/* Report day */
+			    unsigned int hour;		/* Report hour */
+			    unsigned int minute;	/* Report minute */
+			    bool vislimit;		/* Max range? */
+			    unsigned int visibility;	/* Units of 0.1 nm */
+#define DAC8FID21_VISIBILITY_NOT_AVAILABLE	127
+#define DAC8FID21_VISIBILITY_SCALE		10.0
+			    unsigned humidity;		/* units of 1% */
+			    unsigned int wspeed;	/* average wind speed */
+			    unsigned int wgust;		/* wind gust */
+#define DAC8FID21_WSPEED_NOT_AVAILABLE		127
+			    unsigned int wdir;		/* wind direction */
+#define DAC8FID21_WDIR_NOT_AVAILABLE		360
+			    unsigned int pressure;	/* air pressure, hpa */
+#define DAC8FID21_NONWMO_PRESSURE_NOT_AVAILABLE	403
+#define DAC8FID21_NONWMO_PRESSURE_HIGH		402	/* > 1200hPa */
+#define DAC8FID21_NONWMO_PRESSURE_OFFSET		400	/* N/A */
+			    unsigned int pressuretend;	/* tendency */
+		    	    int airtemp;		/* temp, units 0.1C */
+#define DAC8FID21_AIRTEMP_NOT_AVAILABLE		-1024
+#define DAC8FID21_AIRTEMP_SCALE			10.0
+			    unsigned int watertemp;	/* units 0.1degC */
+#define DAC8FID21_WATERTEMP_NOT_AVAILABLE	501
+#define DAC8FID21_WATERTEMP_SCALE		10.0
+			    unsigned int waveperiod;	/* in seconds */
+#define DAC8FID21_WAVEPERIOD_NOT_AVAILABLE	63
+			    unsigned int wavedir;	/* direction in deg */
+#define DAC8FID21_WAVEDIR_NOT_AVAILABLE		360
+			    unsigned int swellheight;	/* in decimeters */
+			    unsigned int swellperiod;	/* in seconds */
+			    unsigned int swelldir;	/* direction in deg */
+			} nonwmo_obs;
+			struct {
+			    signed int lon;		/* Longitude */
+			    signed int lat;		/* Latitude */
+			    unsigned int month;		/* UTC month */
+			    unsigned int day;		/* Report day */
+			    unsigned int hour;		/* Report hour */
+			    unsigned int minute;	/* Report minute */
+			    unsigned int course;	/* course over ground */
+			    unsigned int speed;		/* speed, m/s */
+#define DAC8FID21_SOG_NOT_AVAILABLE		31
+#define DAC8FID21_SOG_HIGH_SPEED		30
+#define DAC8FID21_SOG_SCALE			2.0
+			    unsigned int heading;	/* true heading */
+#define DAC8FID21_HDG_NOT_AVAILABLE		127
+#define DAC8FID21_HDG_SCALE			5.0
+			    unsigned int pressure;	/* units of hPa * 0.1 */
+#define DAC8FID21_WMO_PRESSURE_SCALE		10
+#define DAC8FID21_WMO_PRESSURE_OFFSET		90.0
+			    unsigned int pdelta;	/* units of hPa * 0.1 */
+#define DAC8FID21_PDELTA_SCALE			10
+#define DAC8FID21_PDELTA_OFFSET			50.0
+			    unsigned int ptend;		/* enumerated */
+			    unsigned int twinddir;	/* in 5 degree steps */
+#define DAC8FID21_TWINDDIR_NOT_AVAILABLE	127
+			    unsigned int twindspeed;	/* meters per second */
+#define DAC8FID21_TWINDSPEED_SCALE		2
+#define DAC8FID21_RWINDSPEED_NOT_AVAILABLE	255
+			    unsigned int rwinddir;	/* in 5 degree steps */
+#define DAC8FID21_RWINDDIR_NOT_AVAILABLE	127
+			    unsigned int rwindspeed;	/* meters per second */
+#define DAC8FID21_RWINDSPEED_SCALE		2
+#define DAC8FID21_RWINDSPEED_NOT_AVAILABLE	255
+			    unsigned int mgustspeed;	/* meters per second */
+#define DAC8FID21_MGUSTSPEED_SCALE		2
+#define DAC8FID21_MGUSTSPEED_NOT_AVAILABLE	255
+			    unsigned int mgustdir;	/* in 5 degree steps */
+#define DAC8FID21_MGUSTDIR_NOT_AVAILABLE	127
+			    unsigned int airtemp;	/* degress K */
+#define DAC8FID21_AIRTEMP_OFFSET		223
+			    unsigned humidity;		/* units of 1% */
+#define DAC8FID21_HUMIDITY_NOT_VAILABLE		127
+			    /* some trailing fields are missing */
+			} wmo_obs;
+		    };
+	        } dac1fid21;
+		/*** WORK IN PROGRESS ENDS HERE ***/
 		/* IMO289 - Dangerous Cargo Indication */
 		struct {
 		    unsigned int unit;	/* Unit of Quantity */
@@ -1113,9 +1211,9 @@ struct ais_t
 		struct {
 #define DAC1FID11_LATLON_SCALE			1000
 		    int lon;			/* longitude in minutes * .001 */
-#define DAC1FID11_LON_NOT_AVAILABLE		0x1FFFFFF
-		    int lat;			/* longitude in minutes * .001 */
-#define DAC1FID11_LAT_NOT_AVAILABLE		0xFFFFFF
+#define DAC1FID11_LON_NOT_AVAILABLE		0xFFFFFF
+		    int lat;			/* latitude in minutes * .001 */
+#define DAC1FID11_LAT_NOT_AVAILABLE		0x7FFFFF
 		    unsigned int day;		/* UTC day */
 		    unsigned int hour;		/* UTC hour */
 		    unsigned int minute;	/* UTC minute */
@@ -1125,24 +1223,33 @@ struct ais_t
 		    unsigned int wdir;		/* wind direction */
 		    unsigned int wgustdir;	/* wind gust direction */
 #define DAC1FID11_WDIR_NOT_AVAILABLE		511
-		    int airtemp;		/* temperature, units 0.1C */
-#define DAC1FID11_AIRTEMP_NOT_AVAILABLE		1447
+		    unsigned int airtemp;	/* temperature, units 0.1C */
+#define DAC1FID11_AIRTEMP_NOT_AVAILABLE		2047
+#define DAC1FID11_AIRTEMP_OFFSET		600
+#define DAC1FID11_AIRTEMP_DIV			10.0
 		    unsigned int humidity;	/* relative humidity, % */
 #define DAC1FID11_HUMIDITY_NOT_AVAILABLE	127
-		    int dewpoint;		/* dew point, units 0.1C */
-#define DAC1FID11_DEWPOINT_NOT_AVAILABLE	823
+		    unsigned int dewpoint;	/* dew point, units 0.1C */
+#define DAC1FID11_DEWPOINT_NOT_AVAILABLE	1023
+#define DAC1FID11_DEWPOINT_OFFSET		200
+#define DAC1FID11_DEWPOINT_DIV		10.0
 		    unsigned int pressure;	/* air pressure, hpa */
-#define DAC1FID11_PRESSURE_NOT_AVAILABLE	1311
+#define DAC1FID11_PRESSURE_NOT_AVAILABLE	511
+#define DAC1FID11_PRESSURE_OFFSET		-800
 		    unsigned int pressuretend;	/* tendency */
 #define DAC1FID11_PRESSURETREND_NOT_AVAILABLE	3
 		    unsigned int visibility;	/* units 0.1 nautical miles */
 #define DAC1FID11_VISIBILITY_NOT_AVAILABLE	255
+#define DAC1FID11_VISIBILITY_DIV		10.0
 		    int waterlevel;		/* decimeters */
-#define DAC1FID11_WATERLEVEL_NOT_AVAILABLE	411
+#define DAC1FID11_WATERLEVEL_NOT_AVAILABLE	511
+#define DAC1FID11_WATERLEVEL_OFFSET		100
+#define DAC1FID11_WATERLEVEL_DIV		10.0
 		    unsigned int leveltrend;	/* water level trend code */
-#define DAC1FID11_LEVELTREND_NOT_AVAILABLE	3
+#define DAC1FID11_WATERLEVELTREND_NOT_AVAILABLE	3
 		    unsigned int cspeed;	/* surface current speed in deciknots */
 #define DAC1FID11_CSPEED_NOT_AVAILABLE		255
+#define DAC1FID11_CSPEED_DIV			10.0
 		    unsigned int cdir;		/* surface current dir., degrees */
 #define DAC1FID11_CDIR_NOT_AVAILABLE		511
 		    unsigned int cspeed2;	/* current speed in deciknots */
@@ -1154,6 +1261,7 @@ struct ais_t
 		    unsigned int cdepth3;	/* measurement depth, m */
 		    unsigned int waveheight;	/* in decimeters */
 #define DAC1FID11_WAVEHEIGHT_NOT_AVAILABLE	255
+#define DAC1FID11_WAVEHEIGHT_DIV		10.0
 		    unsigned int waveperiod;	/* in seconds */
 #define DAC1FID11_WAVEPERIOD_NOT_AVAILABLE	63
 		    unsigned int wavedir;	/* direction in degrees */
@@ -1163,31 +1271,34 @@ struct ais_t
 		    unsigned int swelldir;	/* direction in degrees */
 		    unsigned int seastate;	/* Beaufort scale, 0-12 */
 #define DAC1FID11_SEASTATE_NOT_AVAILABLE	15
-		    int watertemp;		/* units 0.1deg Celsius */
-#define DAC1FID11_WATERTEMP_NOT_AVAILABLE	923
+		    unsigned int watertemp;	/* units 0.1deg Celsius */
+#define DAC1FID11_WATERTEMP_NOT_AVAILABLE	1023
+#define DAC1FID11_WATERTEMP_OFFSET		100
+#define DAC1FID11_WATERTEMP_DIV		10.0
 		    unsigned int preciptype;	/* 0-7, enumerated */
 #define DAC1FID11_PRECIPTYPE_NOT_AVAILABLE	7
 		    unsigned int salinity;	/* units of 0.1ppt */
 #define DAC1FID11_SALINITY_NOT_AVAILABLE	511
+#define DAC1FID11_SALINITY_DIV		10.0
 		    unsigned int ice;		/* is there sea ice? */
 #define DAC1FID11_ICE_NOT_AVAILABLE		3
 		} dac1fid11;
 		/* IMO236 - Fairway Closed */
 		struct {
-		    char reason[20+1];	/* Reason For Closing */
+		    char reason[20+1];		/* Reason For Closing */
 		    char closefrom[20+1];	/* Location Of Closing From */
-		    char closeto[20+1];	/* Location of Closing To */
+		    char closeto[20+1];		/* Location of Closing To */
 		    unsigned int radius;	/* Radius extension */
 #define AIS_DAC1FID13_RADIUS_NOT_AVAILABLE 10001
 		    unsigned int extunit;	/* Unit of extension */
 #define AIS_DAC1FID13_EXTUNIT_NOT_AVAILABLE 0
-		    unsigned int fday;	/* From day (UTC) */
+		    unsigned int fday;		/* From day (UTC) */
 		    unsigned int fmonth;	/* From month (UTC) */
-		    unsigned int fhour;	/* From hour (UTC) */
+		    unsigned int fhour;		/* From hour (UTC) */
 		    unsigned int fminute;	/* From minute (UTC) */
-		    unsigned int tday;	/* To day (UTC) */
+		    unsigned int tday;		/* To day (UTC) */
 		    unsigned int tmonth;	/* To month (UTC) */
-		    unsigned int thour;	/* To hour (UTC) */
+		    unsigned int thour;		/* To hour (UTC) */
 		    unsigned int tminute;	/* To minute (UTC) */
 		} dac1fid13;
 	        /* IMO236 - Extended ship and voyage data */
@@ -1258,35 +1369,44 @@ struct ais_t
 		    unsigned int wgustdir;	/* wind gust direction */
 #define DAC1FID31_DIR_NOT_AVAILABLE		360
 		    int airtemp;		/* temperature, units 0.1C */
-#define DAC1FID31_AIRTEMP_NOT_AVAILABLE		-1084
+#define DAC1FID31_AIRTEMP_NOT_AVAILABLE		-1024
+#define DAC1FID31_AIRTEMP_DIV			10.0
 		    unsigned int humidity;	/* relative humidity, % */
 #define DAC1FID31_HUMIDITY_NOT_AVAILABLE	101
 		    int dewpoint;		/* dew point, units 0.1C */
 #define DAC1FID31_DEWPOINT_NOT_AVAILABLE	501
+#define DAC1FID31_DEWPOINT_DIV		10.0
 		    unsigned int pressure;	/* air pressure, hpa */
 #define DAC1FID31_PRESSURE_NOT_AVAILABLE	511
 #define DAC1FID31_PRESSURE_HIGH			402
+#define DAC1FID31_PRESSURE_OFFSET		-799
 		    unsigned int pressuretend;	/* tendency */
-#define DAC1FID31_PRESSURETREND_NOT_AVAILABLE	3
+#define DAC1FID31_PRESSURETEND_NOT_AVAILABLE	3
 		    bool visgreater;            /* visibility greater than */
 		    unsigned int visibility;	/* units 0.1 nautical miles */
 #define DAC1FID31_VISIBILITY_NOT_AVAILABLE	127
-		    int waterlevel;		/* decimeters or cm */
-#define DAC1FID31_WATERLEVEL_NOT_AVAILABLE	40001
+#define DAC1FID31_VISIBILITY_DIV		10.0
+		    int waterlevel;		/* cm */
+#define DAC1FID31_WATERLEVEL_NOT_AVAILABLE	4001
+#define DAC1FID31_WATERLEVEL_OFFSET		1000
+#define DAC1FID31_WATERLEVEL_DIV		100.0
 		    unsigned int leveltrend;	/* water level trend code */
-#define DAC1FID31_LEVELTREND_NOT_AVAILABLE	3
+#define DAC1FID31_WATERLEVELTREND_NOT_AVAILABLE	3
 		    unsigned int cspeed;	/* current speed in deciknots */
 #define DAC1FID31_CSPEED_NOT_AVAILABLE		255
+#define DAC1FID31_CSPEED_DIV			10.0
 		    unsigned int cdir;		/* current dir., degrees */
 		    unsigned int cspeed2;	/* current speed in deciknots */
 		    unsigned int cdir2;		/* current dir., degrees */
 		    unsigned int cdepth2;	/* measurement depth, 0.1m */
 #define DAC1FID31_CDEPTH_NOT_AVAILABLE		301
+#define DAC1FID31_CDEPTH_SCALE			10.0
 		    unsigned int cspeed3;	/* current speed in deciknots */
 		    unsigned int cdir3;		/* current dir., degrees */
 		    unsigned int cdepth3;	/* measurement depth, 0.1m */
 		    unsigned int waveheight;	/* in decimeters */
 #define DAC1FID31_HEIGHT_NOT_AVAILABLE		31
+#define DAC1FID31_HEIGHT_DIV			10.0
 		    unsigned int waveperiod;	/* in seconds */
 #define DAC1FID31_PERIOD_NOT_AVAILABLE		63
 		    unsigned int wavedir;	/* direction in degrees */
@@ -1296,11 +1416,15 @@ struct ais_t
 		    unsigned int seastate;	/* Beaufort scale, 0-12 */
 #define DAC1FID31_SEASTATE_NOT_AVAILABLE	15
 		    int watertemp;		/* units 0.1deg Celsius */
-#define DAC1FID31_PRECIPTYPE_NOT_AVAILABLE	7
+#define DAC1FID31_WATERTEMP_NOT_AVAILABLE	601
+#define DAC1FID31_WATERTEMP_DIV		10.0
 		    unsigned int preciptype;	/* 0-7, enumerated */
-		    unsigned int salinity;	/* units of 0.1% */
+#define DAC1FID31_PRECIPTYPE_NOT_AVAILABLE	7
+		    unsigned int salinity;	/* units of 0.1 permil (ca. PSU) */
 #define DAC1FID31_SALINITY_NOT_AVAILABLE	510
-		    bool ice;			/* is there sea ice? */
+#define DAC1FID31_SALINITY_DIV		10.0
+		    unsigned int ice;		/* is there sea ice? */
+#define DAC1FID31_ICE_NOT_AVAILABLE		3
 		} dac1fid31;
 	    };
 	} type8;
@@ -1373,7 +1497,7 @@ struct ais_t
 	/* Type 17 - GNSS Broadcast Binary Message */
 	struct {
 	    //unsigned int spare;	spare bit(s) */
-#define AIS_GNSS_LATLON_SCALE	600.0
+#define AIS_GNSS_LATLON_DIV	600.0
 	    int lon;			/* longitude */
 	    int lat;			/* latitude */
 	    //unsigned int spare2;	spare bit(s) */
@@ -1473,7 +1597,7 @@ struct ais_t
 	    unsigned int channel_b;	/* Channel B number */
 	    unsigned int txrx;		/* transmit/receive mode */
 	    bool power;			/* high-power flag */
-#define AIS_CHANNEL_LATLON_SCALE	600.0
+#define AIS_CHANNEL_LATLON_DIV	600.0
 	    union {
 		struct {
 		    int ne_lon;		/* NE corner longitude */
@@ -1548,7 +1672,7 @@ struct ais_t
 	    bool accuracy;		/* position accuracy */
 	    bool raim;			/* RAIM flag */
 	    unsigned int status;	/* navigation status */
-#define AIS_LONGRANGE_LATLON_SCALE	600.0
+#define AIS_LONGRANGE_LATLON_DIV	600.0
 	    int lon;			/* longitude */
 #define AIS_LONGRANGE_LON_NOT_AVAILABLE	0x1a838
 	    int lat;			/* latitude */
