@@ -707,6 +707,13 @@ static bool add_device(const char *device_name)
 {
     struct gps_device_t *devp;
     bool ret = false;
+    /* we can't handle paths longer than GPS_PATH_MAX, so don't try */
+    if (strlen(device_name) >= GPS_PATH_MAX) {
+	gpsd_report(LOG_ERROR,
+		    "ignoring device %s: path length exceeds maximum %d\n",
+		    device_name, GPS_PATH_MAX);
+	return false;
+    }
     /* stash devicename away for probing when the first client connects */
     for (devp = devices; devp < devices + MAXDEVICES; devp++)
 	if (!allocated_device(devp)) {
@@ -720,7 +727,7 @@ static bool add_device(const char *device_name)
 
 	    /* do not start more than one ntp thread */
 	    if (!(devp->shmindex >= 0))
-		ntpd_link_activate(devp);
+	        ntpd_link_activate(devp);
 
 	    gpsd_report(LOG_INF, "NTPD ntpd_link_activate: %d\n",
 			(int)devp->shmindex >= 0);
@@ -1498,7 +1505,8 @@ static void consume_packets(struct gps_device_t *device)
 	/* conditional prevents mask dumper from eating CPU */
 	if (context.debug >= LOG_DATA)
 	    gpsd_report(LOG_DATA,
-			"packet from %s with %s\n",
+			"packet type %d from %s with %s\n",
+			device->packet.type,
 			device->gpsdata.dev.path,
 			gps_maskdump(device->gpsdata.set));
 
@@ -1712,6 +1720,7 @@ static void ship_pps_drift_message(struct gps_device_t *session,
 				   struct timeval *tv)
 /* on PPS interrupt, ship a drift message to all clients */
 {
+#ifdef SOCKET_EXPORT_ENABLE
     struct timeval clocktime;
 
     if (gettimeofday(&clocktime, NULL) == 0)
@@ -1719,6 +1728,7 @@ static void ship_pps_drift_message(struct gps_device_t *session,
 			session->gpsdata.dev.path,
 			tv->tv_sec, tv->tv_usec,
 			clocktime.tv_sec, clocktime.tv_usec);
+#endif /* SOCKET_EXPORT_ENABLE */
 }
 #endif /* PPS_ENABLE */
 
@@ -2074,12 +2084,12 @@ int main(int argc, char *argv[])
 	    if (stat(argv[i], &stb) == 0)
 		(void)chmod(argv[i], stb.st_mode | S_IRGRP | S_IWGRP);
 	/*
-	 * Drop privileges.  Up to now we've been running as root.  Instead,
-	 * set the user ID to 'nobody' (or whatever the --enable-gpsd-user
-	 * is) and the group ID to the owning group of a prototypical TTY
-	 * device. This limits the scope of any compromises in the code.
-	 * It requires that all GPS devices have their group read/write
-	 * permissions set.
+	 * Drop privileges.  Up to now we've been running as root.
+	 * Instead, set the user ID to 'nobody' (or whatever the gpsd
+	 * user set by thre build is) and the group ID to the owning
+	 * group of a prototypical TTY device. This limits the scope
+	 * of any compromises in the code.  It requires that all GPS
+	 * devices have their group read/write permissions set.
 	 */
 	/*@-type@*/
 #ifdef GPSD_GROUP
