@@ -25,12 +25,9 @@
 #endif /* S_SPLINT_S */
 
 #include "gpsd.h"
-#if defined(NMEA2000_ENABLE) && !defined(S_SPLINT_S)
-#include <linux/can.h>
-#include <linux/can/raw.h>
-#include <net/if.h>
-#include <sys/ioctl.h>
-#endif /* defined(NMEA2000_ENABLE) && !defined(S_SPLINT_S) */
+#if defined(NMEA2000_ENABLE)
+#include "driver_nmea2000.h"
+#endif /* defined(NMEA2000_ENABLE) */
 
 static void gpsd_run_device_hook(char *device_name, char *hook)
 {
@@ -232,7 +229,7 @@ int gpsd_open(struct gps_device_t *session)
 	return session->gpsdata.gps_fd;
     /* otherwise, could be an TCP data feed */
     } else if (strncmp(session->gpsdata.dev.path, "tcp://", 6) == 0) {
-	char server[GPS_PATH_MAX], *port;
+	char server[strlen(session->gpsdata.dev.path)], *port;
 	socket_t dsock;
 	(void)strlcpy(server, session->gpsdata.dev.path + 6, sizeof(server));
 	session->gpsdata.gps_fd = -1;
@@ -255,7 +252,7 @@ int gpsd_open(struct gps_device_t *session)
 	return session->gpsdata.gps_fd;
     /* or could be UDP */
     } else if (strncmp(session->gpsdata.dev.path, "udp://", 6) == 0) {
-	char server[GPS_PATH_MAX], *port;
+	char server[strlen(session->gpsdata.dev.path)], *port;
 	socket_t dsock;
 	(void)strlcpy(server, session->gpsdata.dev.path + 6, sizeof(server));
 	session->gpsdata.gps_fd = -1;
@@ -281,7 +278,7 @@ int gpsd_open(struct gps_device_t *session)
 #ifdef PASSTHROUGH_ENABLE
     if (strncmp(session->gpsdata.dev.path, "gpsd://", 7) == 0) {
 	/*@-branchstate -nullpass@*/
-	char server[GPS_PATH_MAX], *port;
+	char server[strlen(session->gpsdata.dev.path)], *port;
 	socket_t dsock;
 	(void)strlcpy(server, session->gpsdata.dev.path + 7, sizeof(server));
 	session->gpsdata.gps_fd = -1;
@@ -306,55 +303,7 @@ int gpsd_open(struct gps_device_t *session)
 #endif /* PASSTHROUGH_ENABLE */
 #if defined(NMEA2000_ENABLE) && !defined(S_SPLINT_S)
     if (strncmp(session->gpsdata.dev.path, "nmea2000://", 11) == 0) {
-	char interface_name[GPS_PATH_MAX];
-	socket_t sock;
-	int status;
-	struct ifreq ifr;
-	struct sockaddr_can addr;
-
-	session->gpsdata.gps_fd = -1;
-	/* Create the socket */
-	sock = socket(PF_CAN, SOCK_RAW, CAN_RAW);
- 
-	if (sock == -1) {
-	    gpsd_report(LOG_ERROR, "NMEA2000 open: can not get socket.\n");
-	    return -1;
-	}
-
-	status = fcntl(sock, F_SETFL, O_NONBLOCK);
-	if (status != 0) {
-	    gpsd_report(LOG_ERROR, "NMEA2000 open: can not set socket to O_NONBLOCK.\n");
-	    close(sock);
-	    return -1;
-	}
-
-	(void)strlcpy(interface_name, session->gpsdata.dev.path + 11, sizeof(interface_name));
-	/* Locate the interface you wish to use */
-	strlcpy(ifr.ifr_name, interface_name, sizeof(ifr.ifr_name));
-	status = ioctl(sock, SIOCGIFINDEX, &ifr); /* ifr.ifr_ifindex gets filled 
-						   * with that device's index */
-
-	if (status != 0) {
-	    gpsd_report(LOG_ERROR, "NMEA2000 open: can not find CAN device.\n");
-	    close(sock);
-	    return -1;
-	}
-
-	/* Select that CAN interface, and bind the socket to it. */
-	addr.can_family = AF_CAN;
-	addr.can_ifindex = ifr.ifr_ifindex;
-	status = bind(sock, (struct sockaddr*)&addr, sizeof(addr) );
-	if (status != 0) {
-	    gpsd_report(LOG_ERROR, "NMEA2000 open: bind failed.\n");
-	    close(sock);
-	    return -1;
-	}
-
-	gpsd_switch_driver(session, "NMEA2000");
-	session->gpsdata.gps_fd = sock;
-	session->sourcetype = source_nmea2000;
-	session->servicetype = service_nmea2000;
-	return session->gpsdata.gps_fd;
+        return nmea2000_open(session);
     }
 #endif /* defined(NMEA2000_ENABLE) && !defined(S_SPLINT_S) */
     /* fall through to plain serial open */
@@ -1058,14 +1007,14 @@ gps_mask_t gpsd_poll(struct gps_device_t *session)
 
 	/*
 	 * The guard looks superfluous, but it keeps the rather expensive
-	 * gpsd_hexdump() function from being called even when the debug
+	 * gpsd_packetdump() function from being called even when the debug
 	 * level does not actually require it.
 	 */
 	if (session->context->debug >= LOG_RAW)
 	    gpsd_report(LOG_RAW, "raw packet of type %d, %zd:%s\n",
 			session->packet.type,
 			session->packet.outbuflen,
-			gpsd_hexdump((char *)session->packet.outbuffer, session->packet.outbuflen));
+			gpsd_packetdump((char *)session->packet.outbuffer, session->packet.outbuflen));
 
 	/* Get data from current packet into the fix structure */
 	if (session->packet.type != COMMENT_PACKET)
