@@ -412,6 +412,8 @@ static void nextstate(struct gps_packet_t *lexer, unsigned char c)
     case NMEA_BANG:
 	if (c == 'A')
 	    lexer->state = AIS_LEAD_1;
+	else if (c == 'B')
+	    lexer->state = AIS_LEAD_ALT1;
 	else
 	    lexer->state = GROUND_STATE;
 	break;
@@ -427,6 +429,18 @@ static void nextstate(struct gps_packet_t *lexer, unsigned char c)
 	else
 	    lexer->state = GROUND_STATE;
 	break;
+    case AIS_LEAD_ALT1:
+	if (c == 'S')
+	    lexer->state = AIS_LEAD_ALT2;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case AIS_LEAD_ALT2:
+	if (isalpha(c))
+	    lexer->state = NMEA_LEADER_END;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
 #if defined(TNT_ENABLE) || defined(GARMINTXT_ENABLE) || defined(ONCORE_ENABLE)
     case AT1_LEADER:
 	switch (c) {
@@ -1348,7 +1362,7 @@ static void packet_accept(struct gps_packet_t *lexer, int packet_type)
 	if (lexer->debug >= LOG_RAW+1)
 	    gpsd_report(LOG_RAW+1, "Packet type %d accepted %zu = %s\n",
 		    packet_type, packetlen,
-			gpsd_hexdump((char *)lexer->outbuffer, lexer->outbuflen));
+			gpsd_packetdump((char *)lexer->outbuffer, lexer->outbuflen));
     } else {
 	gpsd_report(LOG_ERROR, "Rejected too long packet type %d len %zu\n",
 		    packet_type, packetlen);
@@ -1366,7 +1380,7 @@ static void packet_discard(struct gps_packet_t *lexer)
 	gpsd_report(LOG_RAW + 1,
 		    "Packet discard of %zu, chars remaining is %zu = %s\n",
 		    discard, remaining,
-		    gpsd_hexdump((char *)lexer->inbuffer, lexer->inbuflen));
+		    gpsd_packetdump((char *)lexer->inbuffer, lexer->inbuflen));
 }
 
 static void character_discard(struct gps_packet_t *lexer)
@@ -1377,7 +1391,7 @@ static void character_discard(struct gps_packet_t *lexer)
     if (lexer->debug >= LOG_RAW+1)
 	gpsd_report(LOG_RAW + 1, "Character discarded, buffer %zu chars = %s\n",
 		    lexer->inbuflen,
-		    gpsd_hexdump((char *)lexer->inbuffer, lexer->inbuflen));
+		    gpsd_packetdump((char *)lexer->inbuffer, lexer->inbuflen));
 }
 
 /* get 0-origin big-endian words relative to start of packet buffer */
@@ -1467,6 +1481,10 @@ void packet_parse(struct gps_packet_t *lexer)
 		packet_accept(lexer, AIVDM_PACKET);
 	    else if (strncmp((char *)lexer->inbuffer, "!AIVDO", 6) == 0)
 		packet_accept(lexer, AIVDM_PACKET);
+	    else if (strncmp((char *)lexer->inbuffer, "!BSVDM", 6) == 0)
+		packet_accept(lexer, AIVDM_PACKET);
+	    else if (strncmp((char *)lexer->inbuffer, "!BSVDO", 6) == 0)
+		packet_accept(lexer, AIVDM_PACKET);
 	    else
 #endif /* AIVDM_ENABLE */
 		packet_accept(lexer, NMEA_PACKET);
@@ -2001,7 +2019,7 @@ ssize_t packet_get(int fd, struct gps_packet_t *lexer)
 	    gpsd_report(LOG_RAW + 1,
 			"Read %zd chars to buffer offset %zd (total %zd): %s\n",
 			recvd, lexer->inbuflen, lexer->inbuflen + recvd,
-			gpsd_hexdump((char *)lexer->inbufptr, (size_t) recvd));
+			gpsd_packetdump((char *)lexer->inbufptr, (size_t) recvd));
 	lexer->inbuflen += recvd;
     }
     gpsd_report(LOG_SPIN, "packet_get() fd %d -> %zd (%d)\n",
