@@ -23,6 +23,8 @@
    SIRF_ACK_LEAD_2,	/* seen c of possible SiRF Ack */
    AIS_LEAD_1,		/* seen A of possible marine AIS message */
    AIS_LEAD_2,		/* seen I of possible marine AIS message */
+   AIS_LEAD_ALT1,	/* seen B of possible marine AIS message */
+   AIS_LEAD_ALT2,	/* seen S of possible marine AIS message */
 
    SEATALK_LEAD_1,	/* SeaTalk/Garmin packet leader 'I' */
    WEATHER_LEAD_1,	/* Weather instrument packet leader 'W' */
