@@ -54,7 +54,8 @@ static sourcetype_t gpsd_classify(const char *path)
 	 * ftp://ftp.kernel.org/pub/linux/docs/device-list/devices-2.6+.txt
 	 */
 	int devmajor = major(sb.st_rdev);
-	if (devmajor == 4 || devmajor == 204)
+        /* 207 are Freescale i.MX UARTs (ttymxc*) */
+	if (devmajor == 4 || devmajor == 204 || devmajor == 207)
 	    devtype = source_rs232;
 	else if (devmajor == 188)
 	    devtype = source_usb;
@@ -360,8 +361,10 @@ int gpsd_serial_open(struct gps_device_t *session)
     /*@ +boolops +type @*/
 #ifdef HAVE_BLUEZ
     if (bachk(session->gpsdata.dev.path) == 0) {
-        session->gpsdata.gps_fd = socket(AF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);
         struct sockaddr_rc addr = { 0 };
+        session->gpsdata.gps_fd = socket(AF_BLUETOOTH,
+					 SOCK_STREAM,
+					 BTPROTO_RFCOMM);
         addr.rc_family = AF_BLUETOOTH;
         addr.rc_channel = (uint8_t) 1;
         str2ba(session->gpsdata.dev.path, &addr.rc_bdaddr);
@@ -505,7 +508,7 @@ ssize_t gpsd_write(struct gps_device_t * session, const char *buf, size_t len)
     /* extra guard prevents expensive hexdump calls */
     if (session->context->debug >= LOG_IO)
 	gpsd_report(LOG_IO, "=> GPS: %s%s\n",
-		    gpsd_hexdump((char *)buf, len), ok ? "" : " FAILED");
+		    gpsd_packetdump((char *)buf, len), ok ? "" : " FAILED");
     return status;
 }
 
@@ -515,7 +518,7 @@ ssize_t gpsd_write(struct gps_device_t * session, const char *buf, size_t len)
  * MAX_PACKET_LENGTH or we risk never syncing up at all.  Large values
  * will produce annoying startup lag.
  */
-#define SNIFF_RETRIES	256
+#define SNIFF_RETRIES	(MAX_PACKET_LENGTH + 128)
 
 bool gpsd_next_hunt_setting(struct gps_device_t * session)
 /* advance to the next hunt setting  */
