@@ -722,8 +722,11 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 			subp->sub4_18.DN, subp->sub4_18.lsf);
 
 #ifdef NTPSHM_ENABLE
-		if ((subp->sub4_18.WNt == subp->sub4_18.WNlsf) &&
-		    (subp->sub4_18.DN == 1 )) {
+		/* IS-GPS-200 Revision E, paragraph 20.3.3.5.2.4 */
+		if (((session->context->gps_week % 256) == (unsigned short)subp->sub4_18.WNlsf) &&
+		    /* notify the leap seconds correction in the end of current day */
+		    ((double)((subp->sub4_18.DN - 1) * SECS_PER_DAY) < session->context->gps_tow) &&
+		    ((double)(subp->sub4_18.DN * SECS_PER_DAY) > session->context->gps_tow)) {
 		   if ( subp->sub4_18.leap < subp->sub4_18.lsf )
 			session->context->leap_notify = LEAP_ADDSECOND;
 		   else if ( subp->sub4_18.leap > subp->sub4_18.lsf )
